///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                         Hi-Res interval timer code                        //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit D_timer;

{
	Should get 840nS resolution, but the graininess is quite coarse.
	(ie ticks are every 840nS, but the accuracy is much less )
}

interface

uses
	Windows;

type { useful for monitoring average frame speed }
	frame_speed_obj = Class(TObject)
		average_frame_speed : double;
		averages            : double;
		procedure reset(avgs:double; av_speed:double);
		function  update(newtime:double) : double;
	end;

procedure start_ticker;
procedure stop_ticker;
function  tick_seconds    : double;
function  tick_seconds_so_far   : double;
function  elapsed_ticks   : longint;
procedure get_ticks;

implementation

var
	ticks_per_second : comp;
	last_ticks       : comp;
	this_ticks       : comp;

{ --------------------------------------------------------------------------- }
{           Frame speed object                                                }
{ --------------------------------------------------------------------------- }
procedure frame_speed_obj.reset(avgs:double; av_speed:double);
begin
	average_frame_speed := av_speed;
	averages            := avgs;
end;

function frame_speed_obj.update(newtime:double) : double;
begin
	average_frame_speed := average_frame_speed - (average_frame_speed/averages);
	average_frame_speed := average_frame_speed + (newtime/averages);
	result := average_frame_speed;
end;
{ --------------------------------------------------------------------------- }
{           Timer/Ticker stuff                                                }
{ --------------------------------------------------------------------------- }
procedure start_ticker;
begin
	get_ticks;
	last_ticks := this_ticks;
end;

procedure stop_ticker;
begin
	get_ticks;
end;

function elapsed_ticks : longint;
begin
	elapsed_ticks := round(this_ticks-last_ticks);
end;

function tick_seconds : double;
begin
	if this_ticks>last_ticks then tick_seconds := (this_ticks-last_ticks)/Ticks_per_second
	else begin  { timer overflow }
		tick_seconds := (9.223372036854775807E18-(last_ticks-this_ticks)+1)/Ticks_per_second;
	end;
end;

{ returns time elapsed without resetting ticker }
function tick_seconds_so_far : double;
var temp : comp;
begin
	temp := this_ticks;
	get_ticks;
	tick_seconds_so_far := tick_seconds;
	this_ticks    := temp;
end;

procedure get_ticks;
begin
	QueryPerformanceCounter(TLargeInteger(this_ticks));
end;

initialization
	QueryPerformanceFrequency(TLargeInteger(ticks_per_second));
  { should be 840 nS - or 838.xxx or something }
end.

