///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                           Main Zonerings window                           //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Form_Main;

interface

uses
	{ Borland }
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, Math,
	{ mine }
	Zone_rings, Zone_settings, PalUnit, Thinkbox, D_timer, HiResTim, ExtCtrls;

type
  TZonewindow = class(TForm)
    Change_timer: TTimer;
    Frame_timer: THiResTimer;
		procedure FormResize(Sender:TObject);
		procedure FormClick(Sender:TObject);
    procedure FormMouseDown(Sender:TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure FormPaint(Sender:TObject);
		procedure FormMouseMove(Sender:TObject; Shift: TShiftState; X,
			Y: Integer);
		procedure FormDestroy(Sender:TObject);
		procedure FormShow(Sender:TObject);
		//
		procedure Change_timer_event(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		// objects we'll need
		Zone_instance : Zonerings;
		// variables for speed control...
		paused        : boolean;
		ready					: boolean;
		// if error message needed...
		calculated  : boolean;
		textrect    : TRect;
		dx,dy,ex,ey : integer;
		Procedure NewSetting(pick_random:boolean);
		function  are_settings_different(CheckStrings:TStringList) : boolean;
		procedure setup_cycling_change;
		function  GetPalette : hPalette; override;
		procedure WM_SysCommand(var message:TWMSysCommand); message WM_SysCommand;
		procedure Frame_timer_event;
		procedure Error_timer_event;
	end;

var
	Zonewindow : TZonewindow; // main black full-screen form

implementation

uses
	Form_Settings, Form_Load_Save, Form_About;

{$R *.DFM}

{ ----------------------------------------------------------------------- }
{                       Form routines                                     }
{ ----------------------------------------------------------------------- }
procedure TZonewindow.FormShow(Sender:TObject);
begin
	SetBounds(0,0,GetSystemMetrics(SM_CXSCREEN) ,GetSystemMetrics(SM_CYSCREEN));
	ready        := false;
	paused       := false;
	calculated   := false;
	if ZOptions.error_flag=[] then NewSetting(true)
	else begin
		Frame_timer.FPS			:= 30;
		Frame_timer.OnTimer	:= Error_timer_event;
		Frame_timer.Enabled	:= true;
	end;
	create_256_pens_using_paletteindex(global_pens,PS_SOLID);
end;

procedure TZonewindow.FormDestroy(Sender:TObject);
begin
	Frame_Timer.Enabled	:= false;
	if Zone_instance<>nil then Zone_instance.Free;
	dispose_256_pens(global_pens);
end;

procedure TZonewindow.WM_SysCommand(var message:TWMSysCommand);
begin
	if message.CmdType=SC_Screensave then message.result:=0
	else inherited;
end;

function TZonewindow.GetPalette : hPalette;
begin
	if Zone_instance<>nil then result := Zone_instance.Zone_palette
	else result := inherited GetPalette;
end;

procedure TZonewindow.FormResize(Sender:TObject);
begin
// only needed when debugging in a sizeable window
//	if Zone_instance<>nil then Zone_instance.Window_Changed(handle);
end;

procedure TZonewindow.FormMouseDown(Sender:TObject; Button: TMouseButton;
	Shift: TShiftState; X, Y: Integer);
begin
	case button of
		mbRight : begin
			paused := not paused;
			if not paused then setup_cycling_change;
		end;
		mbLeft  : ;
	end;
end;

procedure TZonewindow.FormPaint(Sender:TObject);
begin
	if (About_Form<>nil) and (About_Form.Visible) then exit;
	if LoadSave_form.Visible then exit;
	if Zone_instance<>nil then Zone_instance.draw_all_rings;
end;

procedure TZonewindow.FormMouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
begin
	if Zone_instance<>nil then Zone_instance.set_mousepos(x,y);
end;

///////////////////////////////////////////////////////////////////////////////
//   Main bit ..............                                                 //
///////////////////////////////////////////////////////////////////////////////

procedure TZonewindow.FormClick(Sender:TObject);
var TempStrings : TStringList;
begin
	Change_timer.Enabled	:= false;
	Frame_timer.Enabled		:= false;

	if (Zone_instance<>nil) then Settings_Form.f_speed.caption :=
		IntToStr(round(Zone_instance.frame_speed.average_frame_speed));

	TempStrings := nil;
	Settings_Form.WriteText(TempStrings,'Temp setting');
	if Settings_Form.ShowModal=mrCancel then begin
		Settings_Form.ReadText(TempStrings);
		setup_cycling_change;
		Frame_timer.Enabled := true;
	end	else begin
		if ZOptions.Cycling_mode and (not Settings_Form.just_loaded) and are_settings_different(tempStrings) then begin
			case MessageDlg('You have changed some settings, but are running in Auto-cycling mode'#13#10+
											'              Changes will be lost when mode cycles.'#13#10#13#10+
				'Do you want to disable cycling ?', mtConfirmation, [mbYes,mbNo], 0) of
				mrNo     : ;
				mrYes    : ZOptions.Cycling_mode := false;
			end
		end;
		if ZOptions.error_flag=[] then NewSetting(false);
	end;
	TempStrings.Free;
	paused := false;

	if ZOptions.error_flag<>[] then begin
		Frame_timer.FPS			:= 30;
		Frame_timer.OnTimer	:= Error_timer_event;
	end;
end;

function TZonewindow.are_settings_different(CheckStrings:TStringList) : boolean;
var someStrings : TStringList;
		lp1         : integer;
begin
	result := false;
	somestrings := nil;
	Settings_Form.WriteText(somestrings,'Temp setting');
	if somestrings.count=CheckStrings.count then begin
		for lp1:=0 to somestrings.count-1 do begin
			if somestrings[lp1]<>CheckStrings[lp1] then begin
				result := true;
				break;
			end;
		end;
	end else result := true;
	somestrings.Free;
end;

procedure TZonewindow.NewSetting(pick_random:boolean);
begin
	ready := false;
	Frame_Timer.Enabled	:= false;
	if pick_random then
		LoadSave_Form.select_random_setting(ZOptions.Cycle_tagged);
	//
	if (Zone_instance<>nil) then Zone_instance.Free;
	Zone_instance := Zonerings.Create(self,Settings_Form);
	//
	if Settings_Form.hide_cursor.Checked then cursor := 2
	else cursor := 1;

	// setup next change time
	setup_cycling_change;
	//
	if settings_form.fix_speed.Checked then
		Frame_timer.FPS	 	:= Zone_instance.ideal_speed
	else
		Frame_timer.FPS	 	:= 200;
	Frame_timer.OnTimer := Frame_timer_event;
	Frame_Timer.Enabled	:= true;
	//
	paused := false;
	ready  := true;
end;

procedure TZonewindow.setup_cycling_change;
begin
	change_timer.Enabled := false;
	with ZOptions do change_timer.Interval := 1000*(Min_time+random(Max_time-Min_time+1));
	change_timer.Enabled := ZOptions.Cycling_mode;
end;

procedure TZonewindow.Change_timer_event(Sender: TObject);
begin
	NewSetting(true);
end;

procedure TZonewindow.Frame_timer_event;
begin
	if not ready then exit;
	Zone_instance.Update_frame_counter;
	Zone_instance.CycleRings(paused);
end;

procedure TZonewindow.Error_timer_event;
begin
	if (not calculated) then begin
		calculated := true;
		SetRect(textrect,0,0,150,0);
		DrawText(Canvas.Handle,Pchar(ZOptions.error_string),-1,textrect,
			DT_CALCRECT + DT_CENTER + DT_NOCLIP + DT_WORDBREAK);
		// centre the messsage, and pick a random direction
		ex := (Width  - textrect.right) div 2;
		ey := (Height - textrect.bottom) div 2;
		OffsetRect(textrect,ex,ey);
		dx := 0; dy := 0;
		while dx=0 do	dx := random(7) - 3;
		while dy=0 do	dy := random(7) - 3;
		PatBlt(Canvas.Handle,0,0,Width,Height,BLACKNESS);
	end;
	// rub out last message
	SetTextColor(Canvas.Handle,clBlack);
	DrawText(Canvas.Handle,Pchar(ZOptions.error_string),-1,textrect,
		DT_CENTER + DT_NOCLIP + DT_WORDBREAK);
	// pick a new spot
	if ( (textrect.right+dx)>Width )        then dx := -dx
	else if ( (textrect.left+dx)<0 )        then dx := -dx;
	if ( (textrect.top+dy)<0 )              then dy := -dy
	else if ( (textrect.bottom+dy)>Height ) then dy := -dy;
	inc(textrect.left,dx); inc(textrect.right,dx);
	inc(textrect.top,dy);  inc(textrect.bottom,dy);
	// draw the new message
	SetTextColor(Canvas.Handle,clWhite);
	DrawText(Canvas.Handle,Pchar(ZOptions.error_string),-1,textrect,
		DT_CENTER + DT_NOCLIP + DT_WORDBREAK);
end;

end.
