///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                      General purpose Mini-message box                     //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Thinkbox;

// Warning
// Window is created during initialization (see end) - not by Delphi
// Make sure Delphi doesn't add a CreateForm staement for the thinking box

interface

uses
  { Borland }
	SysUtils, WinTypes, WinProcs, Forms, Classes, Controls, ExtCtrls, ComCtrls;

type
  TThinking_box = class(TForm)
		think_msg : TPanel;
		barpanel  : TPanel;
		bar       : TProgressBar;
	private
	{ Private declarations }
	public
	{ Public declarations }
		pausetime : integer;
		constructor Create(AOwner:TComponent); override;
		procedure		CreateParams(var Params:TCreateParams); override;
	end;

var
	thinkbox_active : boolean;

procedure thinking(parent:TForm; mssage:Pchar);
procedure thinking_bar(parent:TForm; mssage:Pchar);
procedure stopped_thinking;
procedure Thinking_error(mess:PChar);
function  progress_bar : TProgressBar;
procedure set_thinking_pause_time(d:integer);
procedure Thinking_pause;

implementation

{$R *.DFM}

uses
	Screen_Utils;

var
	Thinking_box : TThinking_box;
	fsize        : TSize;

procedure create_thinkbox;
begin
  Thinking_box := TThinking_box.Create(Application);
end;

constructor TThinking_box.Create(AOwner:TComponent);
var oldfont  : Hfont;
	screenDC : hDC;
begin
  inherited create(AOwner);
// Use fixed pitch system font so we can calculate width/height quickly from size
	screenDC:=GetDC(0);
	oldfont:=SelectObject(screenDC,think_msg.Font.Handle);
	GetTextExtentPoint32(screenDC,'8',1,fsize); // all letters are this size
	SelectObject(screenDC,oldfont);
	ReleaseDC(0,screenDC);
	pausetime := 500;
end;

procedure TThinking_box.CreateParams(var Params:TCreateParams);
begin
	inherited CreateParams(Params);
	Params.WindowClass.Style := Params.WindowClass.Style or CS_SAVEBITS;
	// this stops a redraw by saving background bitmap - useful for tiny windows
	// Doesn't seem to work though :(
end;

function  progress_bar : TProgressBar;
begin
	if Thinking_box=nil then create_thinkbox;
	result := Thinking_box.bar;
end;

procedure Thinking(parent:TForm; mssage:Pchar);
var BoxPos,BoxOff : TPoint;
		w,h,l         : integer;
begin
	if Thinking_box=nil then create_thinkbox;
	l := StrLen(mssage);
	w:=(l*fsize.cx)+12; h:=(fsize.cy)+6;
	if parent<>nil then with Parent.ClientRect do begin
		BoxOff.x := (right-left - w) div 2;
		BoxOff.y := (bottom-top - h) div 2;
	end
	else begin
		BoxOff.x := (Screen.width - w) div 2;
		BoxOff.y := (Screen.height - h) div 2;
	end;
	Thinking_box.barpanel.visible := false;
	if parent<>nil then BoxPos := Parent.ClientToScreen(Point(0,0))
	else BoxPos := Point(0,0);
	Thinking_box.SetBounds(BoxPos.x + BoxOff.x,BoxPos.y + BoxOff.y,w,h);
	Thinking_box.think_msg.Caption := mssage;
	ShowWindow(Thinking_box.handle,SW_SHOWNOACTIVATE);
	BringWindowToTop(Thinking_box.handle);
  Thinking_box.Visible := true;
  Thinking_box.repaint;
  thinkbox_active := true;
end;

procedure Thinking_Bar(parent:TForm; mssage:Pchar);
var BoxPos,BoxOff : TPoint;
	w,h,l         : integer;
begin
	if Thinking_box=nil then create_thinkbox;
  l := StrLen(mssage);
  w:=(l*fsize.cx)+12; h:=(fsize.cy)+6+24; //24 for bar panel
  if parent<>nil then with Parent.ClientRect do begin
	BoxOff.x := (right-left - w) div 2;
	BoxOff.y := (bottom-top - h) div 2;
  end
  else begin
	BoxOff.x := (Screen.width - w) div 2;
	BoxOff.y := (Screen.height - h) div 2;
  end;
  Thinking_box.barpanel.visible := false;
  Thinking_box.barpanel.height  := 24;
	Thinking_box.bar.Top    := 4;
	Thinking_box.bar.Height := 24-8;
	Thinking_box.bar.Width  := w-8;
	Thinking_box.bar.left   := 4;
  if parent<>nil then BoxPos := Parent.ClientToScreen(Point(0,0))
  else BoxPos := Point(0,0);
  Thinking_box.SetBounds(BoxPos.x + BoxOff.x,BoxPos.y + BoxOff.y,w,h);
  Thinking_box.think_msg.Caption := mssage;
  Thinking_box.barpanel.visible := true;
  ShowWindow(Thinking_box.handle,SW_SHOWNOACTIVATE);
  BringWindowToTop(Thinking_box.handle);
  Thinking_box.Visible := true;
  Thinking_box.repaint;
  thinkbox_active := true;
end;

procedure Stopped_Thinking;
begin
	ShowWindow(Thinking_box.handle,SW_HIDE);
	Thinking_box.barpanel.visible := false;
	thinkbox_active := false;
end;

procedure Thinking_error(mess:PChar);
begin
	Thinking(nil,mess);
	Sleep(Thinking_box.pausetime);
	stopped_thinking;
end;

procedure set_thinking_pause_time(d:integer);
begin
  Thinking_box.pausetime:=d;
end;

procedure Thinking_pause;
begin
  sleep(Thinking_box.pausetime);
end;

initialization
  Thinking_box := nil;
finalization
//  Thinking_box.Free;
end.
